@@ -63,9 +63,10 @@ class TestDeleteNode(fakes.TestDeleteNode):
         ]
         parsed_args = self.check_parser(self.cmd, argslist, verifylist)
 
-        self.websocket.wait_for_message.return_value = {
+        self.websocket.wait_for_messages.return_value = iter([{
+            "execution": {"id": "IDID"},
             "status": "SUCCESS"
-        }
+        }])
 
         self.stack_name.return_value = mock.Mock(stack_name="overcast")
 
@@ -89,10 +90,6 @@ class TestDeleteNode(fakes.TestDeleteNode):
         ]
         parsed_args = self.check_parser(self.cmd, argslist, verifylist)
 
-        self.websocket.wait_for_message.return_value = {
-            "status": "SUCCESS"
-        }
-
         self.stack_name.return_value = None
 
         self.assertRaises(exceptions.InvalidConfiguration,
@@ -112,9 +109,10 @@ class TestDeleteNode(fakes.TestDeleteNode):
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        self.websocket.wait_for_message.return_value = {
+        self.websocket.wait_for_messages.return_value = iter([{
+            "execution": {"id": "IDID"},
             "status": "SUCCESS"
-        }
+        }])
 
         self.cmd.take_action(parsed_args)
 
@@ -137,11 +135,11 @@ class TestDeleteNode(fakes.TestDeleteNode):
         ]
         parsed_args = self.check_parser(self.cmd, argslist, verifylist)
 
-        self.websocket.wait_for_message.return_value = {
+        self.websocket.wait_for_messages.return_value = iter([{
             "status": "FAILED",
             "message": """Failed to run action ERROR: Couldn't find \
                 following instances in stack overcloud: wrong_instance"""
-        }
+        }])
 
         self.assertRaises(exceptions.InvalidConfiguration,
                           self.cmd.take_action, parsed_args)
@@ -168,11 +166,12 @@ class TestProvideNode(fakes.TestOvercloudNode):
         # Get the command object to test
         self.cmd = overcloud_node.ProvideNode(self.app, None)
 
-    def test_provide_all_manageable_nodes(self):
-        self.websocket.wait_for_message.return_value = {
+        self.websocket.wait_for_messages.return_value = iter([{
             "status": "SUCCESS",
-            "message": ""
-        }
+            "message": "Success"
+        }])
+
+    def test_provide_all_manageable_nodes(self):
 
         parsed_args = self.check_parser(self.cmd,
                                         ['--all-manageable'],
@@ -187,11 +186,6 @@ class TestProvideNode(fakes.TestOvercloudNode):
     def test_provide_one_node(self):
         node_id = 'node_uuid1'
 
-        self.websocket.wait_for_message.return_value = {
-            "status": "SUCCESS",
-            "message": "Success"
-        }
-
         parsed_args = self.check_parser(self.cmd,
                                         [node_id],
                                         [('node_uuids', [node_id])])
@@ -207,11 +201,6 @@ class TestProvideNode(fakes.TestOvercloudNode):
         node_id1 = 'node_uuid1'
         node_id2 = 'node_uuid2'
 
-        self.websocket.wait_for_message.return_value = {
-            "status": "SUCCESS",
-            "message": "Success"
-        }
-
         argslist = [node_id1, node_id2]
         verifylist = [('node_uuids', [node_id1, node_id2])]
 
@@ -252,11 +241,11 @@ class TestIntrospectNode(fakes.TestOvercloudNode):
         self.cmd = overcloud_node.IntrospectNode(self.app, None)
 
     def _check_introspect_all_manageable(self, parsed_args, provide=False):
-        self.websocket.wait_for_message.return_value = {
+        self.websocket.wait_for_messages.return_value = iter([{
             "status": "SUCCESS",
             "message": "Success",
             "introspected_nodes": {}
-        }
+        }] * 2)
 
         self.cmd.take_action(parsed_args)
 
@@ -279,11 +268,10 @@ class TestIntrospectNode(fakes.TestOvercloudNode):
                          2 if provide else 1)
 
     def _check_introspect_nodes(self, parsed_args, nodes, provide=False):
-        self.websocket.wait_for_message.return_value = {
+        self.websocket.wait_for_messages.return_value = [{
             "status": "SUCCESS",
             "message": "Success",
-            "introspected_nodes": {}
-        }
+        }]
 
         self.cmd.take_action(parsed_args)
 
@@ -387,13 +375,13 @@ class TestImportNode(fakes.TestOvercloudNode):
 
     def _check_workflow_call(self, parsed_args, introspect=False,
                              provide=False, local=True, no_deploy_image=False):
-        self.websocket.wait_for_message.return_value = {
+        self.websocket.wait_for_messages.return_value = [{
             "status": "SUCCESS",
             "message": "Success",
             "registered_nodes": [{
                 "uuid": "MOCK_NODE_UUID"
             }]
-        }
+        }]
 
         self.cmd.take_action(parsed_args)
 
@@ -488,10 +476,10 @@ class TestConfigureNode(fakes.TestOvercloudNode):
         self.workflow = self.app.client_manager.workflow_engine
         client = self.app.client_manager.tripleoclient
         self.websocket = client.messaging_websocket()
-        self.websocket.wait_for_message.return_value = {
+        self.websocket.wait_for_messages.return_value = iter([{
             "status": "SUCCESS",
             "message": ""
-        }
+        }])
 
         # Get the command object to test
         self.cmd = overcloud_node.ConfigureNode(self.app, None)
@@ -516,10 +504,10 @@ class TestConfigureNode(fakes.TestOvercloudNode):
         )
 
     def test_failed_to_configure_all_manageable_nodes(self):
-        self.websocket.wait_for_message.return_value = {
+        self.websocket.wait_for_messages.return_value = iter([{
             "status": "FAILED",
             "message": "Test failure."
-        }
+        }])
 
         parsed_args = self.check_parser(self.cmd, ['--all-manageable'], [])
         self.assertRaises(exceptions.NodeConfigurationError,
@@ -544,10 +532,10 @@ class TestConfigureNode(fakes.TestOvercloudNode):
         )
 
     def test_failed_to_configure_specified_nodes(self):
-        self.websocket.wait_for_message.return_value = {
+        self.websocket.wait_for_messages.return_value = iter([{
             "status": "FAILED",
             "message": "Test failure."
-        }
+        }])
 
         parsed_args = self.check_parser(self.cmd, ['node_uuid1'], [])
         self.assertRaises(exceptions.NodeConfigurationError,
@@ -651,3 +639,100 @@ class TestConfigureNode(fakes.TestOvercloudNode):
             'tripleo.baremetal.v1.configure',
             workflow_input=self.workflow_input
         )
+
+
+class TestDiscoverNode(fakes.TestOvercloudNode):
+
+    def setUp(self):
+        super(TestDiscoverNode, self).setUp()
+
+        self.workflow = self.app.client_manager.workflow_engine
+        client = self.app.client_manager.tripleoclient
+        self.websocket = client.messaging_websocket()
+
+        self.cmd = overcloud_node.DiscoverNode(self.app, None)
+
+        self.websocket.wait_for_messages.return_value = [{
+            "status": "SUCCESS",
+            "message": "Success",
+            "registered_nodes": [{
+                "uuid": "MOCK_NODE_UUID"
+            }]
+        }]
+
+    def test_with_ip_range(self):
+        argslist = ['--range', '10.0.0.0/24',
+                    '--credentials', 'admin:password']
+        verifylist = [('ip_addresses', '10.0.0.0/24'),
+                      ('credentials', ['admin:password'])]
+
+        parsed_args = self.check_parser(self.cmd, argslist, verifylist)
+        self.cmd.take_action(parsed_args)
+
+        self.workflow.executions.create.assert_called_once_with(
+            'tripleo.baremetal.v1.discover_and_enroll_nodes',
+            workflow_input={'ip_addresses': '10.0.0.0/24',
+                            'credentials': [['admin', 'password']],
+                            'queue_name': mock.ANY,
+                            'kernel_name': 'bm-deploy-kernel',
+                            'ramdisk_name': 'bm-deploy-ramdisk',
+                            'instance_boot_option': 'local'}
+        )
+
+    def test_with_address_list(self):
+        argslist = ['--ip', '10.0.0.1', '--ip', '10.0.0.2',
+                    '--credentials', 'admin:password']
+        verifylist = [('ip_addresses', ['10.0.0.1', '10.0.0.2']),
+                      ('credentials', ['admin:password'])]
+
+        parsed_args = self.check_parser(self.cmd, argslist, verifylist)
+        self.cmd.take_action(parsed_args)
+
+        self.workflow.executions.create.assert_called_once_with(
+            'tripleo.baremetal.v1.discover_and_enroll_nodes',
+            workflow_input={'ip_addresses': ['10.0.0.1', '10.0.0.2'],
+                            'credentials': [['admin', 'password']],
+                            'queue_name': mock.ANY,
+                            'kernel_name': 'bm-deploy-kernel',
+                            'ramdisk_name': 'bm-deploy-ramdisk',
+                            'instance_boot_option': 'local'}
+        )
+
+    def test_with_all_options(self):
+        argslist = ['--range', '10.0.0.0/24',
+                    '--credentials', 'admin:password',
+                    '--credentials', 'admin2:password2',
+                    '--port', '623', '--port', '6230',
+                    '--introspect', '--provide', '--run-validations',
+                    '--no-deploy-image', '--instance-boot-option', 'netboot']
+        verifylist = [('ip_addresses', '10.0.0.0/24'),
+                      ('credentials', ['admin:password', 'admin2:password2']),
+                      ('port', [623, 6230]),
+                      ('introspect', True),
+                      ('run_validations', True),
+                      ('provide', True),
+                      ('no_deploy_image', True),
+                      ('instance_boot_option', 'netboot')]
+
+        parsed_args = self.check_parser(self.cmd, argslist, verifylist)
+        self.cmd.take_action(parsed_args)
+
+        workflows_calls = [
+            mock.call('tripleo.baremetal.v1.discover_and_enroll_nodes',
+                      workflow_input={'ip_addresses': '10.0.0.0/24',
+                                      'credentials': [['admin', 'password'],
+                                                      ['admin2', 'password2']],
+                                      'ports': [623, 6230],
+                                      'queue_name': mock.ANY,
+                                      'kernel_name': None,
+                                      'ramdisk_name': None,
+                                      'instance_boot_option': 'netboot'}),
+            mock.call('tripleo.baremetal.v1.introspect',
+                      workflow_input={'node_uuids': ['MOCK_NODE_UUID'],
+                                      'run_validations': True,
+                                      'queue_name': mock.ANY}),
+            mock.call('tripleo.baremetal.v1.provide',
+                      workflow_input={'node_uuids': ['MOCK_NODE_UUID'],
+                                      'queue_name': mock.ANY}),
+        ]
+        self.workflow.executions.create.assert_has_calls(workflows_calls)
